"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var TextNode_1 = require("../nodes/TextNode");
var dom_1 = require("../utils/dom");
var number_1 = require("../utils/number");
var string_1 = require("../utils/string");
var AbstractElement_1 = require("./AbstractElement");
var InvisibleElement_1 = require("./InvisibleElement");
var stringProperties = [
    'text',
    'title'
];
var insertAtBlacklist = [
    'Ti.UI.TableViewRow'
];
function hasOwnProperty(obj, name) {
    return Object.prototype.hasOwnProperty.call(obj, name);
}
var TitaniumElement = /** @class */ (function (_super) {
    tslib_1.__extends(TitaniumElement, _super);
    function TitaniumElement(tagName, createProxy, meta) {
        var _this = _super.call(this, tagName) || this;
        _this._titaniumProxy = null;
        _this.proxyCreated = false;
        _this.createProxy = createProxy;
        _this.meta = meta;
        return _this;
    }
    Object.defineProperty(TitaniumElement.prototype, "titaniumView", {
        get: function () {
            var _this = this;
            if (this._titaniumProxy === null) {
                var creationProperties_1 = {};
                this.attributes.forEach(function (attributeValue, attributeName) {
                    var propertyName = string_1.camelize(attributeName);
                    _this.setProperty(creationProperties_1, propertyName, attributeValue);
                });
                this._titaniumProxy = this.createProxy(creationProperties_1);
                this.events.forEach(function (handlers, eventName) {
                    handlers.forEach(function (handler) {
                        /* istanbul ignore else */
                        if (_this._titaniumProxy) {
                            Ti.API.debug("Adding event listener for " + eventName + " to created proxy.");
                            _this._titaniumProxy.addEventListener(eventName, handler);
                        }
                    });
                });
                this.proxyCreated = true;
            }
            return this._titaniumProxy;
        },
        enumerable: true,
        configurable: true
    });
    TitaniumElement.prototype.getAttribute = function (name) {
        if (this.proxyCreated === false) {
            return this.getElementAttribute(name);
        }
        var propertyName = string_1.camelize(name);
        /* istanbul ignore else: iOS always returns true for property checks */
        if (Reflect.has(this.titaniumView, propertyName)) {
            return this.titaniumView[propertyName];
        }
        return this.getElementAttribute(name);
    };
    TitaniumElement.prototype.getElementAttribute = function (name) {
        return _super.prototype.getAttribute.call(this, name);
    };
    TitaniumElement.prototype.setAttribute = function (name, value) {
        _super.prototype.setAttribute.call(this, name, value);
        if (this.proxyCreated === false) {
            return;
        }
        var propertyName = string_1.camelize(name);
        this.setProperty(this.titaniumView, propertyName, value);
    };
    /**
     * Reads text from all text child nodes and updates the title or text property of the
     * underlying Titanium view
     */
    TitaniumElement.prototype.updateText = function () {
        var e_1, _a;
        var updatedText = '';
        for (var child = this.firstChild; child !== null; child = child.nextSibling) {
            /* istanbul ignore else */
            if (child instanceof TextNode_1.TextNode) {
                updatedText += child.nodeValue;
            }
        }
        updatedText = updatedText.replace(/^\s+|\s+$/g, '');
        var textPropertyCanditates = ['text', 'title'];
        try {
            for (var textPropertyCanditates_1 = tslib_1.__values(textPropertyCanditates), textPropertyCanditates_1_1 = textPropertyCanditates_1.next(); !textPropertyCanditates_1_1.done; textPropertyCanditates_1_1 = textPropertyCanditates_1.next()) {
                var textProperty = textPropertyCanditates_1_1.value;
                /* istanbul ignore else */
                if (this.proxyCreated && hasOwnProperty(this.titaniumView, textProperty)) {
                    this.setProperty(this._titaniumProxy, textProperty, updatedText);
                    break;
                }
                else {
                    this.setAttribute(textProperty, updatedText);
                }
            }
        }
        catch (e_1_1) { e_1 = { error: e_1_1 }; }
        finally {
            try {
                if (textPropertyCanditates_1_1 && !textPropertyCanditates_1_1.done && (_a = textPropertyCanditates_1.return)) _a.call(textPropertyCanditates_1);
            }
            finally { if (e_1) throw e_1.error; }
        }
    };
    TitaniumElement.prototype.isDetached = function () {
        return this.meta.detached || false;
    };
    TitaniumElement.prototype.shouldDetachChildren = function () {
        return this.meta.detachChildren || false;
    };
    TitaniumElement.prototype.insertBefore = function (newNode, referenceNode) {
        _super.prototype.insertBefore.call(this, newNode, referenceNode);
        if (newNode instanceof TextNode_1.TextNode) {
            this.updateText();
        }
        if (newNode instanceof TitaniumElement) {
            var atIndex = this.getTitaniumChildIndexFromNode(referenceNode);
            this.insertChild(newNode, atIndex);
        }
        if (newNode instanceof InvisibleElement_1.InvisibleElement) {
            var atIndex = this.getTitaniumChildIndexFromNode(referenceNode);
            this.insertIntoVisualTree(newNode, atIndex !== null ? atIndex : this.childElementCount);
        }
    };
    TitaniumElement.prototype.insertIntoVisualTree = function (child, atIndex) {
        var e_2, _a;
        if (child.isDetached() || this.shouldDetachChildren()) {
            return 0;
        }
        if (child instanceof TitaniumElement) {
            this.insertChild(child, atIndex);
            return 1;
        }
        else if (child instanceof InvisibleElement_1.InvisibleElement) {
            var baseIndex = atIndex !== undefined ? atIndex : this.childElementCount;
            var insideIndex = 0;
            var numInserted = 0;
            try {
                for (var _b = tslib_1.__values(child.children), _c = _b.next(); !_c.done; _c = _b.next()) {
                    var childNode = _c.value;
                    numInserted += this.insertIntoVisualTree(childNode, baseIndex + insideIndex++);
                }
            }
            catch (e_2_1) { e_2 = { error: e_2_1 }; }
            finally {
                try {
                    if (_c && !_c.done && (_a = _b.return)) _a.call(_b);
                }
                finally { if (e_2) throw e_2.error; }
            }
            return numInserted;
        }
        return 0;
    };
    TitaniumElement.prototype.removeChild = function (oldChild) {
        _super.prototype.removeChild.call(this, oldChild);
        /* istanbul ignore else */
        if (oldChild instanceof TitaniumElement) {
            /* istanbul ignore if: mainly For type checking only, probably all views have a remove() method */
            if (!this.isContainerView(this.titaniumView)) {
                throw new Error("Unable to remove child " + oldChild + " from parent " + this + " because remove method is unavailable.");
            }
            this.titaniumView.remove(oldChild.titaniumView);
        }
    };
    TitaniumElement.prototype.on = function (eventName, handler) {
        _super.prototype.on.call(this, eventName, handler);
        if (this.proxyCreated) {
            this.titaniumView.addEventListener(eventName, handler);
        }
    };
    TitaniumElement.prototype.off = function (eventName, handler) {
        _super.prototype.off.call(this, eventName, handler);
        if (this.proxyCreated) {
            this.titaniumView.removeEventListener(eventName, handler);
        }
    };
    TitaniumElement.prototype.insertChild = function (element, atIndex) {
        if (element.isDetached() || this.shouldDetachChildren()) {
            return;
        }
        var parentView = this.titaniumView;
        /* istanbul ignore if: mainly for type checking only, probably all views have add methods */
        if (!this.isContainerView(parentView)) {
            throw new Error("Unable to automatically add children to " + this + ". Consider wrapping it in a custom component to manually handle children and make it a detached element.");
        }
        // just assume we have a Ti.UI.View here, add/insertAt will do the actual
        // validation on the native side.
        var childView = element.titaniumView;
        if (atIndex === null || atIndex === undefined || insertAtBlacklist.includes(parentView.apiName)) {
            parentView.add(childView);
        }
        else {
            parentView.insertAt({
                view: childView,
                position: atIndex
            });
        }
    };
    TitaniumElement.prototype.isContainerView = function (view) {
        var requiredFields = ['add', 'insertAt', 'children', 'remove'];
        function isDefined(value) {
            return typeof value !== undefined;
        }
        return requiredFields.every(function (fieldName) { return isDefined(view[fieldName]); });
    };
    TitaniumElement.prototype.getTitaniumChildIndexFromNode = function (node) {
        if (node === null) {
            return null;
        }
        var element = null;
        /* istanbul ignore else: edge case when trying to insert before a non-element node */
        if (node instanceof AbstractElement_1.AbstractElement) {
            element = node;
        }
        else {
            element = node.nextElementSibling;
        }
        if (!element) {
            return null;
        }
        var visualElement;
        try {
            visualElement = dom_1.findSingleVisualElement(element);
        }
        catch (e) {
            return null;
        }
        var childTitaniumView = visualElement.titaniumView;
        if (!this.isContainerView(this.titaniumView)) {
            throw new Error("Unable to determine Titanium child view index for " + visualElement + ". Parent " + this + " is no container view.");
        }
        var childIndex = this.titaniumView.children.indexOf(childTitaniumView);
        if (childIndex !== -1) {
            return childIndex;
        }
        return null;
    };
    TitaniumElement.prototype.setProperty = function (proxy, name, value) {
        if (typeof value === 'string' && !stringProperties.includes(name) && number_1.isNumeric(value)) {
            proxy[name] = number_1.toNumber(value);
        }
        else {
            proxy[name] = value;
        }
    };
    return TitaniumElement;
}(AbstractElement_1.AbstractElement));
exports.TitaniumElement = TitaniumElement;
