import { ElementCollection } from '../ElementCollection';
import { AbstractNode } from './AbstractNode';
import { ChildNodeInterface } from './ChildNodeInterface';
import { ParentNodeInterface } from './ParentNodeInterface';
export declare type EventCallback = (event: any) => any;
/**
 * Represents a default element inside our vdom.
 *
 * Indivudal elements inherit from this class and add features in their
 * implementation.
 */
export declare class ElementNode extends AbstractNode implements ChildNodeInterface, ParentNodeInterface {
    tagName: string;
    attributes: Map<string, any>;
    events: Map<string, Set<EventCallback>>;
    styles: Map<string, any>;
    private _children;
    constructor(tagName: string);
    get innerHTML(): string;
    get outerHTML(): string;
    get childElementCount(): number;
    get firstElementChild(): ElementNode | null;
    get lastElementChild(): ElementNode | null;
    get nextElementSibling(): ElementNode | null;
    get previousElementSibling(): ElementNode | null;
    get children(): ElementCollection;
    insertBefore(newChild: AbstractNode, referenceChild: AbstractNode | null): void;
    getAttribute(name: string): any;
    setAttribute(name: string, value: any): void;
    removeAttribute(name: string): void;
    hasAttribute(name: string): boolean;
    setAttributeNS(namespace: string, name: string, value: any): void;
    removeAttributeNS(namespace: string, name: string): void;
    getStyle(propertyName: string): any;
    setStyle(propertyName: string, value: any): void;
    on(eventName: string, handler: EventCallback): void;
    off(eventName: string, handler: EventCallback): void;
    remove(): void;
    removeChild(oldChild: AbstractNode): void;
    updateText(): void;
    toString(): string;
}
