"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var ElementCollection_1 = require("../ElementCollection");
var device_1 = require("../utils/device");
var AbstractNode_1 = require("./AbstractNode");
/**
 * Represents a default element inside our vdom.
 *
 * Indivudal elements inherit from this class and add features in their
 * implementation.
 */
var ElementNode = /** @class */ (function (_super) {
    tslib_1.__extends(ElementNode, _super);
    function ElementNode(tagName) {
        var _this = _super.call(this) || this;
        _this._children = null;
        _this.nodeName = _this.tagName = tagName.toUpperCase();
        _this.nodeType = AbstractNode_1.NodeType.Element;
        _this.attributes = new Map();
        _this.events = new Map();
        _this.styles = new Map();
        return _this;
    }
    Object.defineProperty(ElementNode.prototype, "innerHTML", {
        get: function () {
            return '';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ElementNode.prototype, "outerHTML", {
        get: function () {
            var tagName = this.tagName.toLowerCase();
            return "<" + tagName + "></" + tagName + ">";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ElementNode.prototype, "childElementCount", {
        get: function () {
            return this.children.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ElementNode.prototype, "firstElementChild", {
        get: function () {
            for (var child = this.firstChild; child !== null; child = child.nextSibling) {
                if (child.nodeType === AbstractNode_1.NodeType.Element) {
                    return child;
                }
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ElementNode.prototype, "lastElementChild", {
        get: function () {
            for (var child = this.lastChild; child !== null; child = child.previousSibling) {
                if (child.nodeType === AbstractNode_1.NodeType.Element) {
                    return child;
                }
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ElementNode.prototype, "nextElementSibling", {
        get: function () {
            for (var child = this.nextSibling; child !== null; child = child.nextSibling) {
                if (child.nodeType === AbstractNode_1.NodeType.Element) {
                    return child;
                }
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ElementNode.prototype, "previousElementSibling", {
        get: function () {
            for (var child = this.previousSibling; child !== null; child = child.previousSibling) {
                if (child.nodeType === AbstractNode_1.NodeType.Element) {
                    return child;
                }
            }
            return null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ElementNode.prototype, "children", {
        get: function () {
            if (!this._children) {
                this._children = new ElementCollection_1.ElementCollection(this);
            }
            return this._children;
        },
        enumerable: true,
        configurable: true
    });
    ElementNode.prototype.insertBefore = function (newChild, referenceChild) {
        _super.prototype.insertBefore.call(this, newChild, referenceChild);
        this.children.invalidateCache();
    };
    ElementNode.prototype.getAttribute = function (name) {
        return this.attributes.get(name);
    };
    ElementNode.prototype.setAttribute = function (name, value) {
        this.attributes.set(name, value);
    };
    ElementNode.prototype.removeAttribute = function (name) {
        this.attributes.delete(name);
    };
    ElementNode.prototype.hasAttribute = function (name) {
        return this.attributes.has(name);
    };
    ElementNode.prototype.setAttributeNS = function (namespace, name, value) {
        if (device_1.runs(namespace)) {
            this.setAttribute(name, value);
        }
    };
    ElementNode.prototype.removeAttributeNS = function (namespace, name) {
        if (device_1.runs(namespace)) {
            this.removeAttribute(name);
        }
    };
    ElementNode.prototype.getStyle = function (propertyName) {
        return this.styles.get(propertyName);
    };
    ElementNode.prototype.setStyle = function (propertyName, value) {
        this.styles.set(propertyName, value);
    };
    ElementNode.prototype.on = function (eventName, handler) {
        var eventHandlers = this.events.get(eventName);
        if (!eventHandlers) {
            eventHandlers = new Set();
            this.events.set(eventName, eventHandlers);
        }
        eventHandlers.add(handler);
    };
    ElementNode.prototype.off = function (eventName, handler) {
        var eventHandlers = this.events.get(eventName);
        if (eventHandlers) {
            eventHandlers.delete(handler);
        }
    };
    ElementNode.prototype.remove = function () {
        if (!this.parentNode) {
            return;
        }
        this.parentNode.removeChild(this);
    };
    ElementNode.prototype.removeChild = function (oldChild) {
        _super.prototype.removeChild.call(this, oldChild);
        this.children.invalidateCache();
    };
    ElementNode.prototype.updateText = function () {
        // Overriden by actual element implementations
    };
    ElementNode.prototype.toString = function () {
        return this.constructor.name + "(" + this.tagName + ")";
    };
    return ElementNode;
}(AbstractNode_1.AbstractNode));
exports.ElementNode = ElementNode;
