"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var string_1 = require("../utils/string");
function normalizeTagName(name) {
    return string_1.camelize(name).toLowerCase();
}
var TitaniumElementRegistry = /** @class */ (function () {
    function TitaniumElementRegistry() {
        this.defaultViewMetadata = {};
        this.elements = new Map();
    }
    TitaniumElementRegistry.getInstance = function () {
        if (!this._instance) {
            this._instance = new TitaniumElementRegistry();
        }
        return this._instance;
    };
    TitaniumElementRegistry.prototype.registerElement = function (options) {
        var meta = options.meta, resolveFactory = options.resolveFactory, elementClass = options.elementClass;
        var tagName = normalizeTagName(options.tagName);
        if (!this.hasElement(tagName)) {
            this.elements.set(tagName.toLowerCase(), {
                resolveFactory: resolveFactory,
                meta: Object.assign({}, this.defaultViewMetadata, meta),
                elementClass: elementClass
            });
        }
        else if (typeof Ti !== 'undefined') {
            Ti.API.warn("Element <" + tagName + "> already registered. Unregister the current one before trying to register it again.");
        }
    };
    TitaniumElementRegistry.prototype.unregisterElement = function (tagName) {
        tagName = normalizeTagName(tagName);
        this.ensureElementIsRegistered(tagName);
        this.elements.delete(tagName);
    };
    TitaniumElementRegistry.prototype.getElement = function (tagName) {
        tagName = normalizeTagName(tagName);
        this.ensureElementIsRegistered(tagName);
        return this.elements.get(tagName);
    };
    TitaniumElementRegistry.prototype.hasElement = function (tagName) {
        tagName = normalizeTagName(tagName);
        return this.elements.has(tagName);
    };
    TitaniumElementRegistry.prototype.getViewFactory = function (tagName) {
        tagName = normalizeTagName(tagName);
        this.ensureElementIsRegistered(tagName);
        return this.getElement(tagName).resolveFactory();
    };
    TitaniumElementRegistry.prototype.getViewMetadata = function (tagName) {
        return this.getElement(tagName).meta;
    };
    TitaniumElementRegistry.prototype.setViewMetadata = function (tagName, meta) {
        this.getElement(tagName).meta = meta;
    };
    TitaniumElementRegistry.prototype.ensureElementIsRegistered = function (tagName) {
        if (!this.hasElement(tagName)) {
            throw new Error("No Titanium view registerd for tag " + tagName);
        }
    };
    return TitaniumElementRegistry;
}());
exports.TitaniumElementRegistry = TitaniumElementRegistry;
