var CollectionIndexCache = /** @class */ (function () {
    function CollectionIndexCache(collection) {
        this._nodeCount = 0;
        this._collection = collection;
        this._cachedList = [];
        this._initialized = false;
    }
    Object.defineProperty(CollectionIndexCache.prototype, "nodeCount", {
        get: function () {
            if (!this._initialized) {
                this.initialize();
            }
            return this._nodeCount;
        },
        enumerable: true,
        configurable: true
    });
    CollectionIndexCache.prototype.nodeAt = function (index) {
        if (!this._initialized) {
            this.initialize();
        }
        if (index >= this._nodeCount) {
            throw new Error('Node index out of bounds.');
        }
        return this._cachedList[index];
    };
    CollectionIndexCache.prototype.indexOf = function (node) {
        if (!this._initialized) {
            this.initialize();
        }
        return this._cachedList.indexOf(node);
    };
    CollectionIndexCache.prototype.updateNodeCountAndListCache = function () {
        var nodeCount = 0;
        for (var child = this._collection.collectionBegin(); child !== null; child = this._collection.traverseForward(child)) {
            nodeCount++;
            this._cachedList.push(child);
        }
        this._nodeCount = nodeCount;
    };
    CollectionIndexCache.prototype.invalidate = function () {
        this._cachedList = [];
        this._nodeCount = 0;
        this._initialized = false;
    };
    CollectionIndexCache.prototype.initialize = function () {
        this.updateNodeCountAndListCache();
        this._initialized = true;
    };
    return CollectionIndexCache;
}());
export { CollectionIndexCache };
