import { CollectionIndexCache } from './CollectionIndexCache';
import { NodeIterator } from './NodeIterator';
var ElementCollection = /** @class */ (function () {
    function ElementCollection(rootNode) {
        this._rootNode = rootNode;
        this._indexCache = new CollectionIndexCache(this);
    }
    Object.defineProperty(ElementCollection.prototype, "length", {
        get: function () {
            return this._indexCache.nodeCount;
        },
        enumerable: true,
        configurable: true
    });
    ElementCollection.prototype.item = function (index) {
        return this._indexCache.nodeAt(index);
    };
    ElementCollection.prototype.indexOf = function (node) {
        return this._indexCache.indexOf(node);
    };
    ElementCollection.prototype.collectionBegin = function () {
        return this._rootNode.firstElementChild;
    };
    ElementCollection.prototype.collectionEnd = function () {
        return this._rootNode.lastElementChild;
    };
    ElementCollection.prototype.traverseForward = function (node) {
        return node.nextElementSibling;
    };
    ElementCollection.prototype.invalidateCache = function () {
        this._indexCache.invalidate();
    };
    ElementCollection.prototype[Symbol.iterator] = function () {
        return new NodeIterator(this);
    };
    return ElementCollection;
}());
export { ElementCollection };
