/// <reference types="titanium" />
import { AbstractNode } from '../nodes/AbstractNode';
import { EventCallback } from '../nodes/ElementNode';
import { AbstractElement } from './AbstractElement';
export declare type ProxyFactory<T extends Titanium.Proxy> = (options: any) => T;
export interface ViewMetadata {
    typeName: string;
    detached?: boolean;
    detachChildren?: boolean;
    [key: string]: any;
}
export interface TitaniumElementConstructor<T extends Titanium.Proxy> {
    new (tagName?: string, createProxy?: ProxyFactory<T>, meta?: ViewMetadata): TitaniumElementInterface<T>;
}
export interface TitaniumElementInterface<T extends Titanium.Proxy> extends AbstractElement {
    meta: ViewMetadata;
    titaniumView: T;
}
export declare class TitaniumElement<T extends Titanium.Proxy> extends AbstractElement implements TitaniumElementInterface<T> {
    meta: ViewMetadata;
    private createProxy;
    private _titaniumProxy;
    private proxyCreated;
    constructor(tagName: string, createProxy: ProxyFactory<T>, meta: ViewMetadata);
    get titaniumView(): T;
    getAttribute(name: string): any;
    getElementAttribute(name: string): any;
    setAttribute(name: string, value: any): void;
    /**
     * Reads text from all text child nodes and updates the title or text property of the
     * underlying Titanium view
     */
    updateText(): void;
    isDetached(): boolean;
    shouldDetachChildren(): boolean;
    insertBefore(newNode: AbstractNode, referenceNode: AbstractNode | null): void;
    insertIntoVisualTree(child: AbstractElement, atIndex?: number): number;
    removeChild(oldChild: AbstractNode): void;
    on(eventName: string, handler: EventCallback): void;
    off(eventName: string, handler: EventCallback): void;
    protected insertChild<U extends Titanium.Proxy>(element: TitaniumElement<U>, atIndex?: number | null): void;
    private isContainerView;
    private getTitaniumChildIndexFromNode;
    private setProperty;
}
