import { TitaniumElement } from '../elements/TitaniumElement';
export function findSingleVisualElement(node) {
    if (node instanceof TitaniumElement) {
        return node;
    }
    var visualElement = null;
    try {
        visualElement = findSingleVisualElementRecursive(node.children);
    }
    catch (e) {
        throw new Error("No suitable visual element found within " + node + ". Reason: " + e.message);
    }
    return visualElement;
}
export function findSingleVisualElementNoThrow(node) {
    try {
        return findSingleVisualElement(node);
    }
    catch (e) {
        return null;
    }
}
function findSingleVisualElementRecursive(elements, nestingLevel) {
    if (nestingLevel === void 0) { nestingLevel = 0; }
    if (elements.length === 0) {
        throw new Error("Reached buttom of tree without finding at least one visual element (nesting level: " + nestingLevel + ").");
    }
    if (elements.length > 1) {
        throw new Error("Expected only one elemnt, but found " + elements.length + " (nesting level: " + nestingLevel + ").");
    }
    if (nestingLevel > 50) {
        throw new Error("Couldn't find a visual element after reaching nesting level limit (nesting level: " + nestingLevel + ".");
    }
    var candidateElement = elements.item(0);
    if (candidateElement instanceof TitaniumElement) {
        return candidateElement;
    }
    return findSingleVisualElementRecursive(candidateElement.children, nestingLevel + 1);
}
